import { useEffect, useState } from "react";
import cockpit from "cockpit";
import { RevPiConfigService } from "../revpi-config/RevPiConfigService.js";

/**
 *
 * @param packageName
 * @returns {Promise<boolean>}
 */
export async function hasPackageConflicts (packageName) {
    try {
        await cockpit.spawn(["apt-get", "install", "--dry-run", packageName]);
        return false;
    } catch (error) {
        console.error(`Conflict detected during dry run for package "${packageName}": `, error);
        return true;
    }
}

/**
 *
 * @param packageName
 * @returns {Promise<void>}
 */
export async function installPackage (packageName) {
    try {
        await cockpit.spawn(["apt-get", "install", "-y", packageName], { superuser: "require" });
        RevPiConfigService.getInstance().refresh();
    } catch (error) {
        console.error("Error installing package: ", error);
        throw new Error("Could not install package: ", { cause: error });
    }
}

/**
 *
 * @returns {Promise<void>}
 */
export async function updatePackages () {
    try {
        await cockpit.spawn(["apt-get", "update"], { superuser: "require" });
    } catch (error) {
        console.error("Failed to update packages: ", error);
        throw new Error("Could not update packages: ", { cause: error });
    }
}

/**
 *
 * @param packageName
 * @returns {Promise<boolean>}
 */
export async function isPackageInstalled (packageName) {
    try {
        await cockpit.spawn(["bash", "-c", `dpkg -l | grep -w '^ii  ${packageName}' `]);
        return true;
    } catch (error) {
        console.info(`Info: ${packageName} is not installed.`, error);
        return false;
    }
}

/**
 *
 * @param packageName
 * @returns {(boolean|(function(): Promise<void>)|*)[]}
 */
export function useInstallationStatus (packageName) {
    const [isInstalled, setIsInstalled] = useState(false);
    useEffect(() => {
        async function checkInstallation () {
            const installed = await isPackageInstalled(packageName);
            setIsInstalled(installed);
        }

        checkInstallation();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []); // Empty dependency array ensures this runs only once on mount
    const handleInstallationComplete = async () => {
        const installed = await isPackageInstalled(packageName);
        setIsInstalled(installed);
    };
    return [isInstalled, handleInstallationComplete];
}
