import React from "react";
import cockpit from "cockpit";
import {
    MenuToggle,
    Select,
    SelectList,
    SelectOption,
    TextInputGroup,
    TextInputGroupMain,
    TextInputGroupUtilities
} from "@patternfly/react-core";
import { getAllCountryCodes, getCurrentCountryCode, setNewCountryCode } from "./WlanService.js";
import { Button } from "@patternfly/react-core/dist/esm/components/Button";
import { TimesIcon } from "@patternfly/react-icons";
import { RevPiFlexBase } from "../../common.components.jsx";

const _ = cockpit.gettext;

export class WlanCountrySelector extends React.Component {
    constructor () {
        super();
        this.state = {
            isLoading: true,
            hasError: false,
            allCountryCodes: [],
            currentCountryCode: ""
        };
    }

    componentDidMount () {
        this.loadData();
    }

    async loadData () {
        this.setState({ isLoading: true });
        try {
            const allCountryCodes = await getAllCountryCodes();
            const currentCountryCode = await getCurrentCountryCode();
            this.setState({
                allCountryCodes,
                currentCountryCode
            });
        } catch (e) {
            console.error(e);
            this.setState({ hasError: true });
            this.setState({ isLoading: false });
        } finally {
            this.setState({ isLoading: false });
        }
    }

    async handleWlanCountrySelect (dropdownSelection) {
        this.setState({ isLoading: true });
        try {
            await setNewCountryCode(dropdownSelection);
            this.setState({ currentCountryCode: dropdownSelection });
        } catch (e) {
            console.error("Error while handling new wlan country", e);
            this.setState({
                hasError: true
            });
        } finally {
            this.setState({ isLoading: false });
        }
    }

    render () {
        return (
            <RevPiFlexBase
                item={{
                    title: _("WLAN country"),
                    rebootRequired: true,
                    isLoading: this.state.isLoading,
                    hasError: this.state.hasError
                }}
            >
                {
                    this.state.allCountryCodes.length > 0 &&
                    <SearchSelect
                        options={this.state.allCountryCodes}
                        status={this.state.currentCountryCode}
                        handleSelect={(selection) => this.handleWlanCountrySelect(selection)}
                    />
                }
            </RevPiFlexBase>
        );
    }
}

export const SearchSelect = (props) => {
    const extractCountryCode = (option) => option.split(" ")[0];
    const getFullCountryString = (code) => props.options.find(c => c.includes(code));
    // initial values
    const initialSelectOptions = props.options.map((e) => ({
        value: extractCountryCode(e),
        children: e
    }));
    const initialSelection = props.options.find(option => extractCountryCode(option) === props.status.trim());

    // state
    const [isOpen, setIsOpen] = React.useState(false);
    const [selected, setSelected] = React.useState(initialSelection);
    const [inputValue, setInputValue] = React.useState(initialSelection);
    const [filterValue, setFilterValue] = React.useState("");
    const [selectOptions, setSelectOptions] = React.useState(initialSelectOptions);
    const [focusedItemIndex, setFocusedItemIndex] = React.useState(null);
    const [activeItem, setActiveItem] = React.useState(null);
    const textInputRef = React.useRef();
    React.useEffect(() => {
        let newSelectOptions = initialSelectOptions;
        if (filterValue) {
            newSelectOptions = initialSelectOptions.filter(menuItem => String(menuItem.children)
                .toLowerCase()
                .includes(filterValue.toLowerCase()));
            if (!newSelectOptions.length) {
                newSelectOptions = [{
                    isDisabled: false,
                    children: `No results found for "${filterValue}"`,
                    value: "no results"
                }];
            }
            if (!isOpen) {
                setIsOpen(true);
            }
        }
        setSelectOptions(newSelectOptions);
        setActiveItem(null);
        setFocusedItemIndex(null);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [filterValue]);
    const onToggleClick = () => {
        setIsOpen(!isOpen);
    };
    const handleSelection = (value) => {
        if (props.options.find(o => o.includes(value)) && value.trim() !== "") {
            const fullCountryString = getFullCountryString(value);
            setInputValue(fullCountryString);
            setSelected(fullCountryString);
            props.handleSelect(value);
        }
    };
    const onSelect = (_event, value) => {
        if (value && value !== "no results") {
            setFilterValue("");
            handleSelection(value);
        }
        setIsOpen(false);
        setFocusedItemIndex(null);
        setActiveItem(null);
    };
    const onTextInputChange = (_event, value) => {
        setInputValue(value);
        setFilterValue(value);
    };
    const handleMenuArrowKeys = key => {
        let indexToFocus;
        if (isOpen) {
            if (key === "ArrowUp") {
                if (focusedItemIndex === null || focusedItemIndex === 0) {
                    indexToFocus = selectOptions.length - 1;
                } else {
                    indexToFocus = focusedItemIndex - 1;
                }
            }
            if (key === "ArrowDown") {
                if (focusedItemIndex === null || focusedItemIndex === selectOptions.length - 1) {
                    indexToFocus = 0;
                } else {
                    indexToFocus = focusedItemIndex + 1;
                }
            }
            setFocusedItemIndex(indexToFocus);
            const focusedItem = selectOptions.filter(option => !option.isDisabled)[indexToFocus];
            setActiveItem(`select-typeahead-${focusedItem.value.replace(" ", "-")}`);
        }
    };
    const onInputKeyDown = event => {
        const enabledMenuItems = selectOptions.filter(option => !option.isDisabled);
        const [firstMenuItem] = enabledMenuItems;
        const focusedItem = focusedItemIndex ? enabledMenuItems[focusedItemIndex] : firstMenuItem;
        switch (event.key) {
        case "Enter":
            if (isOpen && focusedItem.value !== "no results") {
                setInputValue(String(focusedItem.children));
                setFilterValue("");
                handleSelection(String(focusedItem.value));
            }
            setIsOpen(prevIsOpen => !prevIsOpen);
            setFocusedItemIndex(null);
            setActiveItem(null);
            break;
        case "Tab":
        case "Escape":
            setIsOpen(false);
            setActiveItem(null);
            break;
        case "ArrowUp":
        case "ArrowDown":
            event.preventDefault();
            handleMenuArrowKeys(event.key);
            break;
        }
    };
    const toggle = toggleRef => (
        <MenuToggle
            ref={toggleRef} variant='typeahead' onClick={onToggleClick}
            isExpanded={isOpen} isFullWidth
        >
            <TextInputGroup isPlain>
                <TextInputGroupMain
                    value={inputValue}
                    onClick={onToggleClick}
                    onChange={onTextInputChange}
                    onKeyDown={onInputKeyDown}
                    id='typeahead-select-input'
                    autoComplete='off'
                    innerRef={textInputRef}
                    placeholder={_("Select country")} {...(activeItem && {
                        "aria-activedescendant": activeItem
                    })}
                    role='combobox'
                    isExpanded={isOpen}
                    aria-controls='select-typeahead-listbox'
                />

                <TextInputGroupUtilities>
                    {!!inputValue &&
                    <Button
                        variant='plain' onClick={() => {
                            handleSelection("");
                            setInputValue("");
                            setFilterValue("");
                            textInputRef?.current?.focus();
                        }} aria-label='Clear input value'
                    >
                        <TimesIcon aria-hidden />
                    </Button>}
                </TextInputGroupUtilities>
            </TextInputGroup>
        </MenuToggle>
    );
    return (
        <Select
            id='typeahead-select' isOpen={isOpen} selected={selected} onSelect={onSelect} onOpenChange={() => {
                setIsOpen(false);
            }} toggle={toggle}
        >
            <SelectList
                id='select-typeahead-listbox' style={{
                    maxHeight: "250px",
                    overflow: "auto"
                }}
            >
                {selectOptions.map((option, index) =>
                    <SelectOption
                        key={option.value || option.children}
                        isFocused={focusedItemIndex === index}
                        className={option.className}
                        onClick={() => handleSelection(option.value)}
                        id={`select-typeahead-${option.value.replace(" ", "-")}`} {...option}
                        ref={null}
                    />)}
            </SelectList>
        </Select>
    );
};
