import cockpit from "cockpit";
import { getConfigData } from "./RevPiConfigData.js";

let _instance = null;

export class RevPiConfigService {
    static getInstance () {
        _instance ??= new RevPiConfigService();
        return _instance;
    }

    constructor () {
        cockpit.event_target(this);

        this.configData = getConfigData();
    }

    refresh () {
        this.dispatchEvent("changed");
    }

    async callRevpiConfig (command, ids = []) {
        return await cockpit.spawn(["/usr/bin/revpi-config", command, ...ids], { superuser: "require" });
    }

    async toggleConfiguration (id, isEnabled) {
        const command = isEnabled ? "disable" : "enable";
        return this.callRevpiConfig(command, [id]);
    }

    getStatus (id, response) {
        if (response.length === 1) {
            switch (response[0]) {
            case "0":
                return "disabled";
            case "1":
                return "enabled";
            case "2":
                return "unavailable";
            default:
                console.log(`Error: Unexpected response: ${response}. Response must be either 0, 1, or 2.`);
            }
        }
        return response[0];
    }

    parseResponseToObject (responseString) {
        // Initialize an array to hold the parsed response objects
        const parsedResponse = [];

        // Split the response string into an array of individual response items
        const responseArray = responseString.trim()
            .split(" ");

        // Store the lengths of the response and the configData
        const nResponses = responseArray.length;
        const nConfigData = this.configData.length;

        // use a counter to keep track of the multi responses per item
        let indexShifts = 0;
        for (let i = 0; i < this.configData.length; i++) {
            // Retrieve the current configuration item
            const item = this.configData[i];

            // Determine the number of response items expected for this configuration item
            const responseItems = item.responseCount ? item.responseCount : 1;

            // Calculate the start and end index for slicing the response array
            const startIndex = i + indexShifts;
            const endIndex = i + indexShifts + responseItems;

            const rawStatus = responseArray.slice(startIndex, endIndex);
            const status = this.getStatus(item.id, rawStatus);
            const responseObject = {
                ...item,
                status,
                isEnabled: status === "enabled",
                isAvailable: status !== "unavailable",
                hasError: false,
                isLoading: false
            };
            parsedResponse.push(responseObject);

            // Shift the index in case more than one response item has been used
            indexShifts += responseItems - 1;
        }

        // Check if the total number of response items matches the number expected based on configuration
        if (nResponses - indexShifts !== nConfigData) {
            throw new Error("Error: An issue occurred with the index shifting logic.");
        }
        return parsedResponse;
    }

    async getRevpiConfigStatus () {
        const ids = this.configData.map((item) => item.id);
        const response = await this.callRevpiConfig("availstat", ids);
        return this.parseResponseToObject(response);
    }
}
